//-----------------------------------------------------------------------------
//
// Copyright 2023 Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related
// to the source code ("Material") are owned by Intel Corporation or its
// suppliers or licensors. Title to the Material remains with Intel
// Corporation or its suppliers and licensors. The Material contains trade
// secrets and proprietary and confidential information of Intel or its
// suppliers and licensors. The Material is protected by worldwide copyright
// and trade secret laws and treaty provisions. No part of the Material may
// be used, copied, reproduced, modified, published, uploaded, posted,
// transmitted, distributed, or disclosed in any way without Intel's prior
// express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual
// property right is granted to or conferred upon you by disclosure or
// delivery of the Materials, either expressly, by implication, inducement,
// estoppel or otherwise. Any license under such intellectual property rights
// must be express and approved by Intel in writing.
//
//-----------------------------------------------------------------------------
// Top-level internal properties for the “Formal verification part-0” practice
//   session
//-------------------------------------

`ifdef ASSERTIONS_EN

  //---------------------------------------------------------------------------
  // no more than one indicator of arbitrating of a transfer from an input 
  //   interface can be high at the same time
  //---------------------------------------------------------------------------
  property pr__not_more_than_one_arb_in_transferring_high_at_the_same_time;
    @(posedge clk) disable iff (!nreset)
      (arb_in0_transferring_c ||
       arb_in1_transferring_c ||
       arb_in2_transferring_c)
        |->
      ($onehot({arb_in0_transferring_c,
                arb_in1_transferring_c,
                arb_in2_transferring_c}));
  endproperty
  as__not_more_than_one_arb_in_transferring_high_at_the_same_time : assert property(pr__not_more_than_one_arb_in_transferring_high_at_the_same_time) else $error("%t: ERROR: ASSERTION FAILURE: %m", $time);
  
  //---------------------------------------------------------------------------
  // FIFO is not written, when the FIFO is full
  //---------------------------------------------------------------------------
  property pr__fifo_not_written_when_full;
    @(posedge clk) disable iff (!nreset)
      (fifo_we_c)
        |->
      (!fifo_full_c);
  endproperty
  as__fifo_not_written_when_full : assert property(pr__fifo_not_written_when_full) else $error("%t: ERROR: ASSERTION FAILURE: %m", $time);

  //---------------------------------------------------------------------------
  // FIFO is not read, when the FIFO is empty
  //---------------------------------------------------------------------------
  property pr__fifo_not_read_when_empty;
    @(posedge clk) disable iff (!nreset)
      (fifo_re_c)
        |->
      (!fifo_empty_c);
  endproperty
  as__fifo_not_read_when_empty : assert property(pr__fifo_not_read_when_empty) else $error("%t: ERROR: ASSERTION FAILURE: %m", $time);

`endif
